#include "misc.h"

void title_init() 
{
    unsigned short *tex;
    
    title_tex = ta->txr_allocate(512*512*2);
    jpeg_to_texture(TITLE_FILE, title_tex, 512, 1);

    y2k_tex = ta->txr_allocate(512*512*2);
    jpeg_to_texture(BACK_FILE, y2k_tex, 512, 1);
}

void intro_one_frame()
{
    poly_hdr_t poly;

    /* Begin opaque polygons */
    ta->begin_render();

    static_one_frame();
    
    /* Begin translucent polygons */
    ta->commit_eol();

/* you must send something, or the opaque polys don't get rendered! */
    ta->poly_hdr_col(&poly, TA_TRANSLUCENT);
    ta->commit_poly_hdr(&poly);

    /* Finish up */
    ta->commit_eol();
    ta->finish_frame();
}

void title_one_frame(unsigned int tex, float a)
{
    poly_hdr_t poly;
    vertex_ot_t vert;

    ta->poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_RGB565_TWID, 512, 512, tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);
    
    vert.r = vert.g = vert.b = 1.0f;
    vert.a = a;
    vert.oa = vert.or = vert.ob = 0.0f;
    vert.flags = TA_VERTEX_NORMAL;
    
    vert.x = 1;
    vert.y = 1;
    vert.z = 0.0101f;
    vert.u = 0.0;
    vert.v = 0.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 640;
    vert.y = 1;
    vert.z = 0.0101f;
    vert.u = 1.0;
    vert.v = 0.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 1;
    vert.y = 480;
    vert.z = 0.0101f;
    vert.u = 0.0;
    vert.v = 1.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 640;
    vert.y = 480;
    vert.z = 0.0101f;
    vert.u = 1.0;
    vert.v = 1.0;
    vert.flags = TA_VERTEX_EOL;
    ta->commit_vertex(&vert, sizeof(vert));
}

void do_title()
{
  float a = 0.0;
  unsigned int start;

  /* 2 seconds static */
  start = thread->jiffies();
  while( (thread->jiffies() - start) < 200)
      intro_one_frame();

  /* fade in title */
  for(a = 0.0; a <= 1.0 ; a += 0.01) {
    ta->begin_render();
    static_one_frame();
    ta->commit_eol();
    title_one_frame(title_tex, a);
    ta->commit_eol();
    ta->finish_frame();
  }

  /* title 2 seconds */
  ta->begin_render();
  ta->commit_eol();
  title_one_frame(title_tex, 1.0);
  ta->commit_eol();
  ta->finish_frame();
  
  thread->sleep(2000);

  /* fade out title */
  for(a = 1.0; a >= 0.0 ; a -= 0.01) {
    ta->begin_render();
    static_one_frame();
    ta->commit_eol();
    title_one_frame(title_tex, a);
    ta->commit_eol();
    ta->finish_frame();
  }

  /* 2 seconds static */
  start = thread->jiffies();
  while( (thread->jiffies() - start) < 200)
      intro_one_frame();
  
  /* fade in npm */
  for(a = 0.0; a <= 1.0 ; a += 0.01) {
    ta->begin_render();
    static_one_frame();
    ta->commit_eol();
    do_rotozoom(a,0);
    ta->commit_eol();
    ta->finish_frame();
  }

  /* npm 3 seconds */
  start = thread->jiffies();
  while( (thread->jiffies() - start) < 300)
  {
      ta->begin_render();
      ta->commit_eol();
      do_rotozoom(1.0,1);
      ta->commit_eol();
      ta->finish_frame();
  }

  /* fade out npm */
  for(a = 1.0; a >= 0.0 ; a -= 0.01) {
    ta->begin_render();
    static_one_frame();
    ta->commit_eol();
    do_rotozoom(a,1);
    ta->commit_eol();
    ta->finish_frame();
  }

  /* 2 seconds static */
  start = thread->jiffies();
  while( (thread->jiffies() - start) < 200)
      intro_one_frame();

  /* fade in y2kode */
  for(a = 0.0; a <= 1.0 ; a += 0.01) {
    ta->begin_render();
    static_one_frame();
    ta->commit_eol();
    shiver(y2k_tex, a, 16.0f);
    ta->commit_eol();
    ta->finish_frame();
  }

  /* y2kode 1 second */
  start = thread->jiffies();
  while( (thread->jiffies() - start) < 100)
  {
      ta->begin_render();
      ta->commit_eol();
      shiver(y2k_tex, 1.0f, 16.0f);
      ta->commit_eol();
      ta->finish_frame();
  }
}

